<?php


////////////////////////////////////////////////
////// SET PREDEFINED STYLES ///////////////////
////////////////////////////////////////////////

function gg_set_predefined_style() {
	if(!isset($_POST['lcwp_nonce']) || !wp_verify_nonce($_POST['lcwp_nonce'], 'lcwp_nonce')) {
        wp_die('Cheating?');
    }
	if(!isset($_POST['style'])) {
        wp_die('data is missing');
    }
	require_once(GG_DIR .'/settings/preset_styles.php');

	$style_data = gg_preset_styles_data($_POST['style']);
	if(empty($style_data)) {
        wp_die('Style not found');
    }
	
	
	// override values
	foreach($style_data as $key => $val) {
		update_option($key, $val);		
	}


	// if is not forcing inline CSS - create static file
	if(!get_option('mg_inline_css')) {
		gg_static::create_frontend_css();
	}
	
	wp_die('success');
}
add_action('wp_ajax_gg_set_predefined_style', 'gg_set_predefined_style');




////////////////////////////////////////////////
////// CREATE WATERMARK CACHE //////////////////
////////////////////////////////////////////////

function gg_create_wm_cache() {
    if(!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'lcwp_nonce')) {wp_die('Cheating?');}
    
	// force PHP debug mode
	ini_set('display_errors', 1);
	ini_set('display_startup_errors', 1);
	error_reporting(E_ALL);	
	
	
	// specific gallery or any gallery?
	$gid = (isset($_POST['gid'])) ? addslashes($_POST['gid']) : false;
	
	if(!$gid) {
		$args = array(
			'post_type' => 'gg_galleries',
			'numberposts' => -1,
			'post_status' => 'publish'
		);
		$galleries = get_posts($args);
	}
	else {
		$galleries = array( get_post($gid));	
	}
	
	
	// perform
	foreach($galleries as $gallery) {
		$gid = $gallery->ID;
		$type = get_post_meta($gid, 'gg_type', true);
		$images = gg_gall_static::get_data($gid);
	
		if(is_array($images)) {
			foreach($images as $img) { 
				$full_src = gg_static::img_src_on_type($img['img_src'], $type);
				gg_static::watermark_image($full_src);
			}
		}	
	}

	wp_die('success');
}
add_action('wp_ajax_gg_create_wm_cache', 'gg_create_wm_cache');




////////////////////////////////////////////////
////// CLEAN WATERMARK CACHE ///////////////////
////////////////////////////////////////////////

function gg_clean_wm_cache() {
	if(!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'lcwp_nonce')) {wp_die('Cheating?');}
    
    // force PHP debug mode
	ini_set('display_errors', 1);
	ini_set('display_startup_errors', 1);
	error_reporting(E_ALL);	
    
    $wp_dirs = wp_upload_dir();
	$cache_dir = trailingslashit($wp_dirs['basedir']) . 'gg_watermarked';
	
	// folder exists?
	if(!@file_exists($cache_dir)) {
		wp_die('success');	
	}
	
	// clean
	foreach(scandir($cache_dir) as $file) {
		$ext = gg_static::stringToExt($file);
		$accepted = array('.jpg', '.jpeg', '.gif', '.png');
		
		if(in_array($ext, $accepted) && file_exists($cache_dir.'/'.$file)) {
			unlink($cache_dir.'/'.$file);
		}	
	}

	wp_die('success');
}
add_action('wp_ajax_gg_clean_wm_cache', 'gg_clean_wm_cache');





//////////////////////////////////////////////////////////////////////////////





////////////////////////////////////////////////
////// ADD COLLECTION TERM /////////////////////
////////////////////////////////////////////////

function gg_add_coll() {
    if(!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_coll_builder')) {wp_die('cheating?');}
    
	if(!isset($_POST['coll_name'])) {wp_die('missing data');}
	$name = $_POST['coll_name'];
	
	$resp = wp_insert_term( $name, 'gg_collections', array( 'slug'=>sanitize_title($name)) );
	
	if(is_array($resp)) {wp_die('success');}
	else {
		$err_mes = $resp->errors['term_exists'][0];
		wp_die($err_mes);
	}
}
add_action('wp_ajax_gg_add_coll', 'gg_add_coll');



////////////////////////////////////////////////
////// LOAD COLLECTIONS LIST ///////////////////
////////////////////////////////////////////////

function gg_coll_list() {
    if(!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_coll_builder')) {wp_die('cheating?');}
    
	$colls = get_terms('gg_collections', 'hide_empty=0');
    $to_return = array();
    
	foreach($colls as $coll) {
		$to_return[] = array('id' => $coll->term_id, 'name' => $coll->name);
	}

	echo json_encode($to_return);
	wp_die();
}
add_action('wp_ajax_gg_get_colls', 'gg_coll_list');




////////////////////////////////////////////////
////// DELETE THE COLLECTION TERM //////////////
////////////////////////////////////////////////

function gg_del_coll() {
	if(!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_coll_builder')) {wp_die('cheating?');}
    
    if(!isset($_POST['coll_id'])) {
        wp_die('missing data');
    }
	$id = addslashes($_POST['coll_id']);
	
	$resp = wp_delete_term($id, 'gg_collections');
	($resp) ? wp_die('success') : wp_die('error during the collection deletion');
}
add_action('wp_ajax_gg_del_coll', 'gg_del_coll');




////////////////////////////////////////////////
////// DISPLAY COLLECTION BUILDER //////////////
////////////////////////////////////////////////

function gg_coll_builder() {
	if(!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_coll_builder')) {wp_die('cheating?');}
    
    if(!isset($_POST['coll_id'])) {
        wp_die('missing data');
    }
	$coll_id = addslashes($_POST['coll_id']);
			
	// item categories list
	$item_cats = get_terms('gg_gall_categories', 'hide_empty=0');
	
	// cat and page selector
	?>
    <h2></h2>

    <div id="gg_grid_builder_cat" class="postbox">
        <h3 class="hndle"><?php _e("Galleries Picker", GG_ML); ?></h3>
        <div class="inside">

            <div class="gg_mainbox_meta">
                <table class="widefat gg_table gg_metabox_table gg_bordernone">
                    <tr>
                        <td>
                            <label><?php _e("Gallery Categories", GG_ML); ?></label>

                            <select name="gg_gall_cats" class="gg_lc_select_dd" autocomplete="off">
                                <option value="all"><?php esc_html_e('Any category', GG_ML) ?></option>

                                <?php 
            foreach($item_cats as $cat) {
                // WPML fix - get original ID
                if (function_exists('icl_object_id') && isset($GLOBALS['sitpress'])) {
                    global $sitepress;
                    $term_id = icl_object_id($cat->term_id, 'gg_gall_categories', true, $sitepress->get_default_language());
                }
                else {
                    $term_id = $cat->term_id;
                }

                echo '<option value="'.$term_id.'">'. $cat->name .'</option>';
            }
                                ?>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <div>
                                <label><?php _e("Select galleries", GG_ML); ?></label>

                                <input type="text" name="gg_coll_gall_search" placeholder="🔎 <?php esc_attr_e('search galleries', GG_ML) ?>" autocomplete="off" />
                                <i class="gg_cgs_del dashicons dashicons-no-alt" title="<?php esc_attr_e('clear search', GG_ML) ?>"></i>

                                <a href="javascript:void(0)" class="gg_cgs_show_all">(<?php _e('expand', GG_ML) ?>)</a>
                            </div>

                            <ul id="gg_coll_gall_picker">
                                <?php 
                                    $post_list = gg_cat_galleries_code('all'); 
        echo (!$post_list) ? '<span>'. __('No galleries found', GG_ML) .' ..</span>' : $post_list;
                                ?>
                            </ul>
                        </td>
                    </tr>
                </table>
            </div>
        </div>
    </div>


    <div class="postbox">
        <h3 class="hndle"><?php _e("Collection Builder", GG_ML) ?></h3>
        <div class="inside">

            <div id="visual_builder_wrap">
                <table id="gg_coll_builder">
                    <tbody>	
                        <?php
                        $coll_data = get_term($coll_id, 'gg_collections');
                        $coll_composition = unserialize($coll_data->description);
                        $coll_galleries = (array)$coll_composition['galleries'];
                        $has_a_gallery = false;

                        foreach( $coll_galleries as $gdata) {
                            $gid = $gdata['id'];
                            $gall_img = gg_coll_static::get_gall_first_img($gid);	

                            if(get_post_status($gid) == 'publish' && $gall_img) {

                                $rand_check 	= (isset($gdata['rand']) && $gdata['rand'] != 0) ? 'checked="checked"' : '';
                                $wmark_check 	= (isset($gdata['wmark']) && $gdata['wmark'] != 0) ? 'checked="checked"' : '';  
                                $filter_check	= (isset($gdata['filters']) && $gdata['filters'] != 0) ? 'checked="checked"' : ''; 
                                $search_check	= (isset($gdata['search']) && $gdata['search'] != 0) ? 'checked="checked"' : ''; 	

                                $link_subj 	= (isset($gdata['link_subj'])) ? $gdata['link_subj'] : 'none'; 
                                $link_val 	= (isset($gdata['link_val'])) ? $gdata['link_val'] : '';
                                $descr 		= (isset($gdata['descr'])) ? $gdata['descr'] : ''; 

                                // custom image
                                $cust_img = '';
                                $cust_img_id = '';
                                $ci_icon_sel_class = '';  

                                if(isset($gdata['cust_img']) && $gdata['cust_img']) {
                                    $cust_img = gg_static::inline_thumb_src($gdata['cust_img'], 500, 500, 70);
                                    $cust_img_id = $gdata['cust_img'];
                                    $ci_icon_sel_class = 'gg_coll_cust_img_sel';   
                                } 

                                $orig_thumb = gg_static::inline_thumb_src($gall_img, 500, 500, 70);
                                $thumb_to_use = (empty($cust_img)) ? $orig_thumb : $cust_img;

                                // categories
                                $gall_cats = gg_coll_static::gallery_cats($gid, 'list', ', ');
                                $gall_cats = (empty($gall_cats)) ? '<em>'. __('No associated categories', GG_ML) .' ..</em>' : '<em class="dashicons dashicons-tag gg_coll_gall_cats_icon" title="'. esc_attr__('Categories', GG_ML) .'"></em> '.$gall_cats;

                                $has_a_gallery = true;
                                echo str_replace(
                                    array(
                                        '%%GALL_ID%%',
                                        '%%THUMB_TO_USE%%', 
                                        '%%CUST-ICON_SEL_CLASS%%', 
                                        '%%CUST_IMG_ID%%', 
                                        '%%ORIG_THUMB%%', 
                                        '%%GALL_CATS%%', 
                                        '%%GALL_TITLE%%', 
                                        '%%RANDOM_CHECK%%', 
                                        '%%FILTER_CHECK%%',
                                        '%%SEARCH_CHECK%%', 
                                        '%%WATERM_CHECK%%',
                                        '%%DESCR%%'
                                    ),
                                    array(
                                        $gid,
                                        $thumb_to_use,
                                        $ci_icon_sel_class,
                                        $cust_img_id,
                                        $orig_thumb,
                                        $gall_cats,
                                        get_the_title($gid),
                                        (isset($gdata['rand']) && $gdata['rand'] != 0) ? 'checked="checked"' : '',
                                        (isset($gdata['filters']) && $gdata['filters'] != 0) ? 'checked="checked"' : '',
                                        (isset($gdata['search']) && $gdata['search'] != 0) ? 'checked="checked"' : '',	
                                        (isset($gdata['wmark']) && $gdata['wmark'] != 0) ? 'checked="checked"' : '',  
                                        esc_textarea($descr),
                                    ),
                                    gg_coll_static::manual_gall_code_block($link_subj, $link_val)
                                );
                            }
                        }
    
                        if(!$has_a_gallery) {
                            echo '
                              <tr>
                                <td colspan="5">
                                    <p>'.__('No selected galleries', GG_ML).' ..</p>
                                </td>
                              </tr>';
                        }
                        ?>
                    </tbody>
                </table>
            </div> 

        </div>
    </div>
	<?php
	wp_die();
}
add_action('wp_ajax_gg_coll_builder', 'gg_coll_builder');






////////////////////////////////////////////////
////// GET GALLERIES FOR A CATEGORY ////////////
////////////////////////////////////////////////

function gg_cat_galleries_code($fnc_cat = false) {	
	if(!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_coll_builder')) {wp_die('cheating?');}
    
    $cat = $fnc_cat;
	$code = '';
	
	// if is not called directly
	if(!$cat) {
		if(!isset($_POST['gallery_cat'])) {wp_die('missing data');}
		$cat = $_POST['gallery_cat'];
	}

	$post_list = gg_coll_static::cat_galleries($cat);	
	if(!$post_list) {
        return false;
    }
	
	foreach($post_list as $post) {
		$code .= '
		<li style="background-image: url('. gg_static::inline_thumb_src($post['img'], 200, 170, 70) .');" data-id="'.$post['id'].'" title="'. esc_attr__('add to collection', GG_ML) .'" data-gg-cats="'.$post['cats'].'" data-gg-img="'. gg_static::inline_thumb_src($post['img'], 500, 500, 70) .'">
			
            <div title="'. esc_attr($post['title']) .'">'. $post['title'] .'</div>
		</li>';
	}

	
	if($fnc_cat == false) {
        wp_die( $code );
    } else {
        return $code;
    }
}
add_action('wp_ajax_gg_cat_galleries_code', 'gg_cat_galleries_code');






////////////////////////////////////////////
////// SAVE COLLECTION CONTENTS ////////////
////////////////////////////////////////////

function gg_save_coll() {	
	if(!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_coll_builder')) {wp_die('cheating?');}
    
    if(!isset($_POST['coll_id'])) {wp_die('missing data');}
	$coll_id = (int)$_POST['coll_id'];
	
	$data = array();
	$to_check = array('gall_list', 'cust_img', 'random_flag', 'filters_flag', 'search_flag', 'wmark_flag', 'link_subj', 'link_val', 'coll_descr');
	foreach($to_check as $tc) {
		$data[$tc] = (!isset($_POST[$tc])) ? '' : $_POST[$tc];	
	}
	
	// create the categories array
	$terms_array = array();
	if(is_array($data['gall_list'])) {
		foreach($data['gall_list'] as $post_id) {
			$pid_terms = wp_get_post_terms($post_id, 'gg_gall_categories', array("fields" => "ids"));
			
			foreach($pid_terms as $pid_term) { 
				$terms_array[] = $pid_term;
			}	
		}
		$terms_array = array_unique($terms_array);
	}
	
	// create the galleries array
	$galleries = array();
	if(is_array($data['gall_list'])) {
		$a = 0;
		foreach($data['gall_list'] as $gid) {
			$galleries[] = array(
				'id' 		=> $gid,
				'cust_img'	=> $data['cust_img'][$a],
				'rand'		=> $data['random_flag'][$a],
				'filters'	=> $data['filters_flag'][$a],
				'search'	=> $data['search_flag'][$a],
				'wmark' 	=> $data['wmark_flag'][$a],
				'link_subj' => $data['link_subj'][$a],
				'link_val' 	=> esc_attr(stripslashes($data['link_val'][$a])),
				'descr'		=> esc_attr(stripslashes($data['coll_descr'][$a])) 
			);	
			$a++;
		}
	}

	// final array
	$coll_arr = array(
		'galleries' => $galleries,
		'categories' => $terms_array
	);
	
	// update the collection term
	$result = wp_update_term($coll_id, 'gg_collections', array(
	  'slug' => uniqid(),
	  'description' => serialize($coll_arr)
	));
	
	
	if(is_wp_error($result)) {echo 'error';}
	else {echo 'success';}	

	wp_die();
}
add_action('wp_ajax_gg_save_coll', 'gg_save_coll');






//////////////////////////////////////////////////////////////////////////////






////////////////////////////////////////////////
//// SAVE WP FILESYSTEM CREDENTIALS (for no "direct" methods) 
////////////////////////////////////////////////

function gg_save_wp_filesys_data() {
	if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'lcwp_ajax')) {
        wp_die('Cheating?');
    };
	
	include_once(GG_DIR .'/classes/gg_local_images.php');
	$fm = new gg_local_images();
	
	// save credentials
	$data = array(
		'hostname',
		'username',
		'password',
		'public_key',
		'private_key',
		'connection_type'
	);
	
	foreach($data as $key) {
		$data[$key] = (isset($_POST[$key])) ? stripslashes($_POST[$key]) : '';
	}

	// try connecting
	$response = $fm->is_ready();
	if($response === true) {
        echo 'success';
        update_option('gg_filesys_creds', $data);
    }
    else {
        echo $response;
    }
	wp_die();
}
add_action('wp_ajax_gg_save_wp_filesys_data', 'gg_save_wp_filesys_data');






////////////////////////////////////////////////
//// SAVE GG ALBUMS BASEPATHS ////////////////// 
////////////////////////////////////////////////

function gg_save_gg_album_basepaths() {
	if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'lcwp_ajax')) {
        wp_die('Cheating?');
    };

	include_once(GG_DIR .'/classes/gg_local_images.php');
	$fm = new gg_local_images();
	
	// get and validate data
    if(
        !isset($_POST['basepath']) || !isset($_POST['baseurl']) || 
        empty($_POST['basepath']) || empty($_POST['baseurl']) ||
        !preg_match('/^(\.\.\/(?:\.\.\/)*)?(?!.*?\/\/)(?!(?:.*\/)?\.+(?:\/|$)).+$/', $_POST['basepath']) || 
        !filter_var($_POST['baseurl'], FILTER_VALIDATE_URL)
    ) {
        wp_die( __('Please insert valid values', GG_ML));
    }

    $basepath   = untrailingslashit(stripslashes($_POST['basepath']));
    $baseurl    = untrailingslashit(stripslashes($_POST['baseurl']));
    
	
    // check path existence
    $fm->folder_path = $basepath;
    
    if($fm->is_ready()) {
        update_option('gg_albums_basepath', $basepath);
        update_option('gg_albums_baseurl', $baseurl);
        
        wp_die('success');
    }
    else {
        wp_die('Folder not found', GG_ML);       
    }
}
add_action('wp_ajax_gg_save_gg_album_basepaths', 'gg_save_gg_album_basepaths');






////////////////////////////////////////////////
////// LIM - CREATE FOLDER ///////////////////// 
////////////////////////////////////////////////

function gg_lim_create_folder() {
	if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_files_manager')) {
        wp_die('Cheating?');
    };
    
	include_once(GG_DIR .'/classes/gg_local_images.php');
	$fm = new gg_local_images();
	
    // startup engine
    $fm_response = $fm->is_ready();
    if($fm_response !== true) {
        wp_die($fm_response);     
    }
    
    // get and validate folder name
    $f_name = (isset($_POST['f_name'])) ? stripslashes((string)$_POST['f_name']) : '';

    if(empty($f_name) || !preg_match('/^[a-zA-Z0-9_-]+$/', $f_name)) {
        wp_die( __('Invalid format', GG_ML));        
    }
    elseif(strlen($f_name) > 150) {
        wp_die( __('Too long', GG_ML));          
    }
    elseif($fm->folder_exists($f_name)) {
        wp_die( __("A folder with same name already exists", GG_ML));          
    }
    
    
    // create
    if($fm->create_folder($f_name)) {
        wp_die('success');    
    } else {
        wp_die( __('Error creating the folder', GG_ML) );    
    }
}
add_action('wp_ajax_gg_lim_create_folder', 'gg_lim_create_folder');






////////////////////////////////////////////////
////// LIM -  DELETE FOLDER //////////////////// 
////////////////////////////////////////////////

function gg_lim_delete_folder() {
	if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_files_manager')) {
        wp_die('Cheating?');
    };
    
	include_once(GG_DIR .'/classes/gg_local_images.php');
	$fm = new gg_local_images();
	
    // startup engine
    $fm_response = $fm->is_ready();
    if($fm_response !== true) {
        wp_die($fm_response);     
    }
    
    // get and validate folder name
    $f_name = (isset($_POST['f_name'])) ? stripslashes((string)$_POST['f_name']) : '';
    if(empty($f_name)) {
        wp_die( __('Folder name missing', GG_ML));        
    }
    
    
    // delete
    $response = $fm->delete($f_name);
    
    if($response === true) {
        wp_die('success');    
    } else {
        wp_die( __('Error creating the folder', GG_ML) );    
    }
}
add_action('wp_ajax_gg_lim_delete_folder', 'gg_lim_delete_folder');






////////////////////////////////////////////////
////// LIM - GET IMAGES //////////////////////// 
////////////////////////////////////////////////

function gg_lim_get_images() {
	if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_files_manager')) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => 'Cheating?'
        )));
    };

	include_once(GG_DIR .'/classes/gg_local_images.php');
	$fm = new gg_local_images();
	
    // startup engine
    $fm_response = $fm->is_ready();
    if($fm_response !== true) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => $fm_response
        )));
    }
    
    // get and validate folder name
    $f_name = (isset($_POST['f_name'])) ? stripslashes((string)$_POST['f_name']) : '';
    if(empty($f_name)) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => __('Folder name missing', GG_ML)
        )));   
    }
    
    
    // get
    $response = $fm->list_files($f_name);
    
    if(is_array($response)) {
        wp_die(json_encode(array(
            'status'   => 'success',
            'images'   => $response
        )));   
    } 
    else {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => $response
        )));   
    }
}
add_action('wp_ajax_gg_lim_get_images', 'gg_lim_get_images');





////////////////////////////////////////////////
////// LIM - MOVE FILES INTO ANOTHER FOLDER //// 
////////////////////////////////////////////////

function gg_lim_move_files() {
	if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_files_manager')) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => 'Cheating?'
        )));
    };
    
	include_once(GG_DIR .'/classes/gg_local_images.php');
	$fm = new gg_local_images();
	
    // startup engine
    $fm_response = $fm->is_ready();
    if($fm_response !== true) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => $fm_response
        ))); 
    }
    
    
    // get and validate folder name
    $old_f_name = (isset($_POST['old_f_name'])) ? stripslashes((string)$_POST['old_f_name']) : '';
    $new_f_name = (isset($_POST['new_f_name'])) ? stripslashes((string)$_POST['new_f_name']) : '';

    if(
        empty($old_f_name) || !preg_match('/^[a-zA-Z0-9_-]+$/', $old_f_name) ||
        empty($new_f_name) || !preg_match('/^[a-zA-Z0-9_-]+$/', $new_f_name)
    ) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => __('Please choose the destination folder', GG_ML)
        )));     
    }
    elseif(!$fm->folder_exists($new_f_name)) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => __("Destination folder not found", GG_ML)
        )));         
    }
    elseif($old_f_name == $new_f_name) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => __("Current and destination folders coincide", GG_ML)
        )));             
    }
    
    
    // get files to move
    $files = (isset($_POST['files'])) ? (array)$_POST['files'] : array();
    if(empty($files)) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => __("No files to move", GG_ML)
        )));          
    }
    
    $problems = array();
    
    foreach($files as $filename) {
        $old_path = $fm->folder_path .'/'. $old_f_name .'/'. $filename;
        $new_path = $fm->folder_path .'/'. $new_f_name .'/'. $filename;
        
        if(!$fm->fs->move($old_path, $new_path)) {
            $problems[] = $filename;    
        }
    }
    
    wp_die(json_encode(array(
        'status'   => 'success',
        'problems' => $problems
    )));
}
add_action('wp_ajax_gg_lim_move_files', 'gg_lim_move_files');





////////////////////////////////////////////////
////// LIM - DELETE FILES ////////////////////// 
////////////////////////////////////////////////

function gg_lim_delete_files() {
	if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_files_manager')) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => 'Cheating?'
        )));
    };
    
	include_once(GG_DIR .'/classes/gg_local_images.php');
	$fm = new gg_local_images();
	
    // startup engine
    $fm_response = $fm->is_ready();
    if($fm_response !== true) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => $fm_response
        )));       
    }


    // get and validate folder name
    $f_name = (isset($_POST['f_name'])) ? stripslashes((string)$_POST['f_name']) : '';
    if(empty($f_name) || !preg_match('/^[a-zA-Z0-9_-]+$/', $f_name)) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => __('Target folder missing', GG_ML)
        )));     
    }

    // get files to delete
    $files = (isset($_POST['files'])) ? (array)$_POST['files'] : array();
    if(empty($files)) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => __("No files to delete", GG_ML)
        )));           
    }
    
    
    $problems = array();
    foreach($files as $filename) {
        if($fm->delete($f_name, $filename) !== true) {
           $problems[] = $filename;
        }
    }
    
    wp_die(json_encode(array(
        'status'   => 'success',
        'problems' => $problems
    )));
}
add_action('wp_ajax_gg_lim_delete_files', 'gg_lim_delete_files');





////////////////////////////////////////////////
////// LIM - UPDATE IMAGES IPTC DATA /////////// 
////////////////////////////////////////////////

function gg_lim_update_images_data() {
	if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_files_manager')) {
        wp_die('Cheating?');
    };
    
    ini_set('display_errors', 1);
	ini_set('display_startup_errors', 1);
	error_reporting(E_ALL);	
    
	include_once(GG_DIR .'/classes/gg_local_images.php');
	$fm = new gg_local_images();
	
    // startup engine
    $fm_response = $fm->is_ready();
    if($fm_response !== true) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => $fm_response
        )));   
    }
    
    // get and validate folder name
    $f_name = (isset($_POST['f_name'])) ? stripslashes((string)$_POST['f_name']) : '';
    if(empty($f_name)) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => __('Folder name missing', GG_ML)
        )));   
    }
    
    
    // get images data (multidimensional array(filename => array(author, title, ec) ) )
    $data = (isset($_POST['img_data'])) ? (array)$_POST['img_data'] : false;
    if(empty($data) || !is_array($data)) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => __('Images data missing', GG_ML)
        )));       
    }
    
    
    // perform
    foreach($data as $filename => $file_data) {
        include_once(GG_DIR .'/classes/iptc.php');
        
        // only JPEG
        if(!in_array( gg_static::stringToExt($filename), array('.jpg', '.jpeg') )) {
            continue;   
        }
        
        $img_path = $fm->folder_path .'/'. $f_name .'/'. $filename;
        $iptc = new Iptc($img_path, $fm->fs);
        
        $iptc->apply_tags(array(
            '080' => $file_data['author'],
            '005' => $file_data['title'],
            '120' => $file_data['descr'],
            '025' => explode(',', $file_data['tags'])
        ));
    }
    
    wp_die(json_encode(array(
        'status'   => 'success',
    )));
}
add_action('wp_ajax_gg_lim_update_images_data', 'gg_lim_update_images_data');





////////////////////////////////////////////////
//// LIM - IMAGE UPLOAD ////////////////////////
////////////////////////////////////////////////

function gg_lim_upload() {
	if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'gg_files_manager')) {
		wp_die(json_encode(array(
            'status'    => 'error', 
            'message'   => "Cheating?"
        )));
	};

    include_once(GG_DIR .'/classes/gg_local_images.php');
	$fm = new gg_local_images();
	
    // startup engine
    $fm_response = $fm->is_ready();
    if($fm_response !== true) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => $fm_response
        )));      
    }
    
    
    // get and validate folder name
    $f_name = (isset($_POST['f_name'])) ? stripslashes((string)$_POST['f_name']) : '';
    if(empty($f_name) || !preg_match('/^[a-zA-Z0-9_-]+$/', $f_name)) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => __('Target folder missing', GG_ML)
        )));     
    }
	
    
    // get and validate file
	if(!isset($_FILES['gg_lim_image'])) {
		wp_die(json_encode(array(
            'status'    =>'error', 
            'message'   => esc_html__("File not found", GG_ML)
        )));	
	}
	$file = $_FILES['gg_lim_image'];
	
	$result = $fm->validate_file($file);
	if($result !== true) {
		wp_die(json_encode(array(
            'status'    =>'error', 
            'message'   => $result
        ))); 	
	}
	
	
	// upload file
	$response = $fm->upload_file($f_name, $file);
	if($response !== true) {
        wp_die(json_encode(array(
            'status'    => 'error',
            'message'   => $response
        )));
	}
	
	wp_die(json_encode(array(
        'status'    => 'success', 
        'img_data'  => $fm->get_file($f_name, $fm->last_uploaded_filename)
    )));	
}
add_action('wp_ajax_gg_lim_upload', 'gg_lim_upload');